package no.nav.eia.dynaregler.helpers;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import no.nav.common.util.FileUtil;
import no.nav.eia.common.constants.TypeDetektorSM2013;
import no.nav.eia.dynaregler.element.GruppeElement;
import no.nav.eia.dynaregler.element.LedetekstElement;
import no.nav.eia.dynaregler.element.RegelElement;
import no.nav.eia.dynaregler.element.SykmeldingParameterElement;
import no.nav.eia.kontroll.verdiobjekt.MeldingInfo;
import no.nav.xml.JDOMTool;

/**
 * 
 * @author Florian van Eizenga (E140074)
 * Denne Factory-klassen behandler grunnlaget: SM2013DynaRegel_x_x.xml, SM2013DynaSpm_x_x.xml og SM2013Ledetekst_x_x.xml
 * Det betyr at filene kan bli: hentet pga filnavn, lagd til i lister og returnert til den kallende.  
 *
 */

public class DynaRegelFactory  {
 
	private static final Logger infolog = Logger.getLogger("infolog");
	private static final Logger errlog = Logger.getLogger("errorlog");
	 
	private static ResourceBundle fRB = ResourceBundle.getBundle("eia3-2013-java-support"); 
	public static final String LEDETEKST_FILNAVN = "EJS.LEDETEKST.FILNAVN";
	public static final String REGEL_FILNAVN = "EJS.REGEL.FILNAVN";
	public static final String SPM_FILNAVN = "EJS.SPM.FILNAVN";
	public static final String FILE_INPUT_PATH = "EJS.INPUT.PATH";
	public static final String XMLSCHEMA_SYKMELDING_2013_NAMESPACE = "EJS.XMLSCHEMA.SYKMELDING2013";
	public static final String XMLSCHEMA_MSGHEAD_NAMESPACE = "EJS.XMLSCHEMA.MSGHEAD";
	public static final String XMLSCHEMA_FELLES_NAMESPACE = "EJS.XMLSCHEMA.FELLESKOMPONENT";
			
	private static final JDOMTool xmlUtil = new JDOMTool();
	
	public static String getStringProperty(String key) {
		
	 		return fRB.getString(key);
	}
	
	private static void logInfo(String entry) {
		
		Date date = new Date();
		infolog.info("**********");
		infolog.info(date.toString());
		infolog.info(entry + "\n");
	}
	
	private static void logError(String entry, Exception e) {
		
		Date date = new Date();		
		errlog.error("**********");
		errlog.error(date.toString());
		errlog.error(e.getMessage());
		errlog.error(entry + "\n", e);
	}
	
	/**
	 * readXMLFile
	 * 
	 * Denne metoden tar imot en XML-fil og lagrer innholdet i et String-objekt.
	 * Dette String-objektet blir returneres.
	 * 
	 * @param fileName
	 * @return en String med XML content (f.eks. ledetekst, dynaregel, dynaspm, sykmeldingparameter)
	 * @throws Exception
	 */
	public static String readXMLFile(String fileName) {
		
		String f2 = fileName;
		
		if(f2.startsWith("file:")) f2 = f2.substring(6);
		
		String result = null;
		
		try {
			result = FileUtil.getInstance().fileContent(f2,FileUtil.getInstance().getISOCharset());			
		}
		
		catch(Exception e) {
		
			logError("Problems in DynaRegelFactory.readXMLFile ", e);
		}
		
		return result;
	}
	
	public static boolean isDynaSvarUtfylt(MeldingInfo mi, String spmId) {
		
		String sm13SykmeldingAsXml = createSM13SykmeldingAsXMLFromMeldingInfo(mi);
		return sm13SykmeldingAsXml.contains(spmId);
	}
	
	public static String createSM13SykmeldingAsXMLFromMeldingInfo(MeldingInfo mi) {
		
		String sm13SykmeldingAsXml = null;
		Element sykmeldingAsElement = null;
		
		try {
			
			sykmeldingAsElement = new TypeDetektorSM2013().getFagmelding(mi.getFellesformatNode());
			
			if (sykmeldingAsElement != null) {
				
				sm13SykmeldingAsXml = xmlUtil.element2string(sykmeldingAsElement, null, null);
			}	
		} 
		catch (Exception e) {
		
			logError("Problems in DynaRegelFactory.createSM13SykmeldingAsXMLFromMeldingInfo ", e);
		}
				
		return sm13SykmeldingAsXml;
	}
	
	/**
	 * getLedetekstListFromFile
	 * 
	 * Denne metoden tar imot en XML-fil (SM2013Ledetekst_1_1.xml), 
	 * og lagrer alle LedetekstElement-ene i en liste. Denne lista blir returneres.
	 * 
	 * @param filnavn
	 * @return alle ledetekstene fra fil
	 * @throws Exception
	 */
	public static ArrayList<LedetekstElement> getLedetekstListFromFile(String filnavn) {
		
		ArrayList<LedetekstElement> list = new ArrayList<LedetekstElement>();
		
		try {
			
			String xmlName = filnavn;
			FileUtil fu = FileUtil.getInstance(); 
			String xml = fu.resourceContent(xmlName,fu.getISOCharset());
			
			Document doc = (xml != null ? xmlUtil.string2document(xml) : null);				
			Element root = (doc != null ? doc.getRootElement() : null);
			
			@SuppressWarnings("unchecked")
			List<Element> children = (root != null ? root.getChildren("Ledetekst") : new ArrayList<Element>());
			
			for(Element child: children) {
					
				LedetekstElement l = new LedetekstElement().newLedetekstElement(child);
					
				if(l != null) {
					
					list.add(l);
				}
			}
		}
		
		catch (Exception e) {
			
			logError("Problems in DynaRegelFactory.getLedetekstListFromFile() ", e);
			list = null;
		}
		
		return list;
	}
	
	/**
	 * getRegelListFromFile
	 * 
	 * Denne metoden tar imot en XML-fil (SM2013DynaRegel_1_0.xml), 
	 * og lagrer alle RegelElement-ene i en liste. Denne lista blir returneres.
	 * 
	 * @param filnavn
	 * @return alle dynareglene fra fil
	 * @throws Exception
	 */
	public static ArrayList<RegelElement> getRegelListFromFile(String filnavn) {
		
		ArrayList<RegelElement> list = new ArrayList<RegelElement>();
		
		try {
			
				String xmlName = filnavn;
				FileUtil fu = FileUtil.getInstance(); 
				String xml = fu.resourceContent(xmlName,fu.getISOCharset());
				
				Document doc = (xml != null ? xmlUtil.string2document(xml) : null);				
				Element root = (doc != null ? doc.getRootElement() : null);
				
				@SuppressWarnings("unchecked")
				List<Element> children = (root != null ? root.getChildren("DynaRegel") : new ArrayList<Element>());
				
				for(Element child: children) {
									
					RegelElement r = new RegelElement().newRegelElement((Element) child);
						
					if(r != null) {
							
						list.add(r);
					} 
				}
		}
		
		catch (Exception e) {
			
			logError("Problems in DynaRegelFactory.getRegelListFromFile() ", e);
			list = null;
		}
		
		return list;
	}	

	/**
	 * getSpmListFromFile
	 * 
	 * Denne metoden tar imot en XML-fil (SM2013DynaSpm_1_0.xml), 
	 * og lagrer alle GruppeElement-ene i en liste. Denne lista blir returneres.
	 * 
	 * @param filnavn
	 * @return alle dynasprsmlene fra fil
	 * @throws Exception
	 */
	public static ArrayList<GruppeElement> getGruppeAndSpmListFromFile(String filnavn) {
		
		ArrayList<GruppeElement> list = new ArrayList<GruppeElement>();
		
		try {
			
				String xmlName = filnavn;
				FileUtil fu = FileUtil.getInstance(); 
				String xml = fu.resourceContent(xmlName,fu.getISOCharset());
				
				Document doc = (xml != null ? xmlUtil.string2document(xml) : null);				
				Element root = (doc != null ? doc.getRootElement() : null);
				
				@SuppressWarnings("unchecked")
				List<Element> children = (root != null? root.getChildren("DynaGruppe") : new ArrayList<Element>());
				
				for(Element child: children) {
									
					GruppeElement g = new GruppeElement().newGruppeElement(child);
						
					if(g != null) {
							
						list.add(g);
					} 
				}
		}
		
		catch (Exception e) {
			
			logError("Problems in DynaRegelFactory.getGruppeAndSpmListFromFile() ", e);
			list = null; 
		}
		
		return list;
	}
	
	/**
	 * getSykmeldingParameterFromFile
	 * 
	 * Denne metoden tar imot en XML-fil (SM2013SykmeldingParameter_1_0.xml), 
	 * og lagrer alle SykmeldingParameterElement-ene i et objekt. Dette objektet blir returneres.
	 * 
	 * !!!EKSTRA INFO: dette objektet er det innputtet for  teste det dynamiske regelverket!!!  
	 * 
	 * @param filnavn
	 * @return beregnte sykmeldingsverdier fra fil
	 * @throws Exception
	 */
	public static SykmeldingParameterElement getSykmeldingParameterFromFile(String filnavn) {
		
		SykmeldingParameterElement sykmeldingParameter = null;
		
		try {
			
			String xml = readXMLFile(filnavn);
			
			Document doc = (xml != null ? xmlUtil.string2document(xml) : null);				
			Element root = (doc != null ? doc.getRootElement() : null);

			Element child = (root != null ? root.getChild("SykmeldingParameter") : null);
			
			if (child != null) {
				
				sykmeldingParameter = new SykmeldingParameterElement().newSykmeldingParameterElement(child); 
			}
	
		}
		
		catch(Exception e) {
			
			logError("Problems in DynaRegelFactory.getSykmeldingParameterFromFile() ", e);
		}
				
		return sykmeldingParameter;
	}
}